package com.github.gkutiel.filter;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

class Engine {

	private final HttpServletRequest req;
	private final HttpServletResponse res;

	public Engine(final HttpServletRequest req, final HttpServletResponse res) {
		this.req = req;
		this.res = res;
	}

	public void invoke(final Class<? extends Filter> filterClass) {
		invoke(null, filterChain(filterClass));
	}

	private List<Class<? extends Filter>> filterChain(final Class<?> c) {
		if (c == null) return new ArrayList<>();
		final List<Class<? extends Filter>> classChain = filterChain(c.getEnclosingClass());
		if (Filter.class.isAssignableFrom(c)) classChain.add(c.asSubclass(Filter.class));
		return classChain;
	}

	private void invoke(final Filter outter, final List<Class<? extends Filter>> chain) {
		if (outter != null) outter.apply();
		if (chain.isEmpty()) return;
		final Class<? extends Filter> filterClass = chain.remove(0);
		invoke(newFilter(outter, filterClass), chain);
	}

	private Filter newFilter(final Filter outter, final Class<? extends Filter> filterClass) {
		try {
			if (outter == null) return new Injector(filterClass.newInstance(), req, res).inject();
			return new Injector(filterClass.getConstructor(outter.getClass()).newInstance(outter), req, res).inject();
		} catch (final Exception e) {
			throw new RuntimeException(e);
		}
	}
}
